from BrickBreaker.Shared import *
from BrickBreaker.pad import Pad


class Ball(GameObject):

    def __init__(self, position, image, game):
        self._game = game
        self._speed = 4
        self._speed_bonus = 1
        self._increment = [2, 2]
        self._direction = [1, 1]
        self._in_motion = 0

        super().__init__(position, GameConstants.BALL_SIZE, image)

    def set_speed(self, new_speed):
        if new_speed > 6:
            new_speed = 6
        self._speed = new_speed
        self._speed_bonus += 1

    def reset_speed(self):
        self.set_speed(4)

    def get_speed(self):
        return self._speed

    def is_in_motion(self):
        return self._in_motion

    def set_motion(self, is_moving):
        self._in_motion = is_moving
        self.reset_speed()

    def change_direction(self, game_object):

        position = self.get_position()
        size = self.get_size()
        object_position = game_object.get_position()
        object_size = game_object.get_size()

        if object_position[1] + object_size[1] > position[1] > object_position[1] and \
                object_position[0] + object_size[0] > position[0] > object_position[0]:
            self.set_position((position[0], object_position[1] + object_size[1]))
            self._direction[1] *= -1

        elif object_position[1] + object_size[1] > position[1] + size[1] > object_position[1] and \
                object_position[0] + object_size[0] > position[0] > object_position[0]:
            self.set_position((position[0], object_position[1] - object_size[1]))
            self._direction[1] *= -1

        elif object_position[0] + object_size[0] > position[0] + size[0] > object_position[0]:
            self.set_position((object_position[0] - size[0], position[1]))
            self._direction[0] *= -1

        else:
            self.set_position((object_position[0] + object_size[0], position[1]))
            self._direction[0] *= -1
            self._direction[1] *= -1

    def update_position(self):
        if not self.is_in_motion():
            pad_position = self._game.get_pad().get_position()
            if isinstance(self._game.get_pad(), Pad):
                self.set_position((
                    pad_position[0] + (GameConstants.PAD_SIZE[0] / 2) - (GameConstants.BALL_SIZE[0] / 2),
                    GameConstants.SCREEN_SIZE[1] - GameConstants.PAD_SIZE[1] - GameConstants.BALL_SIZE[1]
                ))
            else:
                self.set_position((
                    pad_position[0] + (GameConstants.DOUBLE_PAD_SIZE[0] / 2) - (GameConstants.BALL_SIZE[0] / 2),
                    GameConstants.SCREEN_SIZE[1] - GameConstants.DOUBLE_PAD_SIZE[1] - GameConstants.BALL_SIZE[1]
                ))
            return

        position = self.get_position()
        size = self.get_size()

        new_position = [position[0] + (self._increment[0] * self._speed) * self._direction[0],
                        position[1] + (self._increment[1] * self._speed) * self._direction[1]]

        if new_position[0] + size[0] >= GameConstants.SCREEN_SIZE[0]:
            self._direction[0] *= -1
            new_position = [GameConstants.SCREEN_SIZE[0] - size[0], new_position[1]]
            self._game.play_sound(GameConstants.SOUND_HIT_WALL_OR_PAD)

        if new_position[0] <= 0:
            self._direction[0] *= -1
            new_position = [0, new_position[1]]
            self._game.play_sound(GameConstants.SOUND_HIT_WALL_OR_PAD)

        if new_position[1] + size[1] >= GameConstants.SCREEN_SIZE[1]:
            self._direction[1] *= -1
            new_position = [new_position[0], GameConstants.SCREEN_SIZE[1] - size[1]]

        if new_position[1] <= 0:
            self._direction[1] *= -1
            new_position = [new_position[0], 0]
            self._game.play_sound(GameConstants.SOUND_HIT_WALL_OR_PAD)

        self.set_position(new_position)

    def is_ball_dead(self):
        position = self.get_position()
        size = self.get_size()

        if position[1] + size[1] >= GameConstants.SCREEN_SIZE[1]:
            return 1
        return 0



import pygame

from BrickBreaker.Scenes.scene import Scene
from BrickBreaker.Shared import *


class GameRulesScene(Scene):

    def __init__(self, game):
        super().__init__(game)

    def render(self):

        self.clear_text()
        self.add_text("WELCOME TO BRICK BREAKER", x=450, y=50, size=60)
        self.add_text("Each brick is worth 100 points. After hitting speed brick speed of all active balls "
                      "will increase by set amount. You can stack 2 of these buffs per ball.", x=100, y=200, size=30)
        self.add_text("Hitting speed brick will also make all bricks be worth double of their initial value. "
                      "This applies to the extra(bonus) points as well and can be stacked ", x=100, y=250, size=30)
        self.add_text("indefinetly. Both bonus and speed buff will reset upon losing a life. Each life "
                      "brick you hit will increase total amount of your life by one. If you ", x=100, y=300, size=30)
        self.add_text("complete the game every extra life you have left will increase your final score "
                      "by 10 000. Bonus size bricks will double the size of your pad. ", x=100, y=350, size=30)
        self.add_text("These are not stackable. However they will grant you extra 1000 points upon hitting "
                      "them if you are already affected by the buff they provide. Extra ball ", x=100, y=400, size=30)
        self.add_text("bricks will give you an extra ball after you hit them and there is no limit to the amount "
                      "of balls you can have active at one time. If you have more than ", x=100, y=450, size=30)
        self.add_text("one ball active and you miss the ball with your pad that ball will be lost, "
                      "but you won't lose life. You start on level 1 with a single row of ", x=100, y=500, size=30)
        self.add_text("bricks, with each next level increasing the amount of rows, and thus "
                      "the difficulty. There is total of eight levels.", x=100, y=550, size=30)
        # self.add_text("the difficulty. There is total of eight levels.", x=100, y=550, size=30 )
        self.add_text("F1 - Back to Main Menu", x=300, y=800, size=60)

        super().render()

    def handle_events(self, events):
        super().handle_events(events)

        for event in events:
            if event.type == pygame.QUIT:
                exit()

            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    self.get_game().change_scene(GameConstants.MAIN_MENU_SCENE)

                if event.key == pygame.K_F1:
                    self.get_game().change_scene(GameConstants.MAIN_MENU_SCENE)



import pygame

from BrickBreaker.Scenes.scene import Scene
from BrickBreaker.Shared import *
from BrickBreaker import Highscore


class WinScene(Scene):

    def __init__(self, game):
        super().__init__(game)

        self._player_name = ""

    def render(self):

        self._game.screen.fill((120, 0, 120))

        self.clear_text()
        self.add_text("YOU WIN!", x=600, y=200, size=100, color=[242, 111, 0], background=[120, 0, 120],
                      font_type="Ani", italic=True, bold=True)
        self.add_text("F1 - Start New Game", x=300, y=500, background=[120, 0, 120], size=30)
        self.add_text("F2 - Quit", x=1100, y=500, background=[120, 0, 120], size=30)
        self.add_text("Please enter your name: ", x=700, y=700, background=[120, 0, 120], size=30)
        self.add_text(self._player_name, x=700, y=800, background=[120, 0, 120], size=30)

        super().render()

    def handle_events(self, events):
        super().handle_events(events)

        for event in events:
            if event.type == pygame.QUIT:
                exit()

            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    exit()

                if event.key == pygame.K_F1:
                    self.get_game().reset()
                    self.get_game().change_scene(GameConstants.PLAYING_SCENE)

                if event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_RETURN:
                        game = self.get_game()
                        Highscore().add(self._player_name, game.get_score() + game.get_lives() * 10000)
                        game.reset()
                        game.change_scene(GameConstants.HIGHSCORE_SCENE)
                    elif 122 >= event.key >= 65:
                        self._player_name += chr(event.key)

                if event.key == pygame.K_F2:
                    exit()



import fileinput
import hashlib
import operator
from pkg_resources import resource_filename
import getpass
import os


class Highscore:

    def __init__(self):
        self._highscore = self.load()

    def get_scores(self):
        return self._highscore

    @staticmethod
    def load():
        user = getpass.getuser()
        highscore = []
        try:
            f = open(f"/home/{user}/highscore.dat", "r")
        except IOError:
            f = open(f"/home/{user}/highscore.dat", "w")
        for line in fileinput.input(f"/home/{user}/highscore.dat"):
            name, score, md5 = line.split('[::]')
            md5 = md5.replace('\n', '')

            if str(hashlib.md5(str.encode(str(name+score+"pygame"))).hexdigest()) == str(md5):
                highscore.append([str(name), int(score), str(md5)])

        highscore.sort(key=operator.itemgetter(1), reverse=True)
        highscore = highscore[0:10]
        f.close()

        return highscore

    def add(self, name, score):
        user = getpass.getuser()
        hash = hashlib.md5((str(name+str(score)+"pygame")).encode("utf-8"))
        self._highscore.append([name, str(score), hash.hexdigest()])

        with open(f"/home/{user}/highscore.dat", "w") as f:
            for name, score, md5 in self._highscore:
                f.write(str(name)+"[::]"+str(score)+"[::]"+str(md5)+"\n")




from pkg_resources import resource_filename


class GameConstants:

    SCREEN_SIZE = [1600, 900]
    BRICK_SIZE = [SCREEN_SIZE[0] / 16, SCREEN_SIZE[1] / 30]
    BALL_SIZE = [16, 16]
    PAD_SIZE = [139, 13]
    DOUBLE_PAD_SIZE = [278, 13]

    BALL_IMAGE = resource_filename(__name__, "ball.png")
    STANDARD_BRICK_IMAGE = resource_filename(__name__, "standard_brick_try.png")
    SPEED_BRICK_IMAGE = resource_filename(__name__, "speed_brick_try.png")
    LIFE_BRICK_IMAGE = resource_filename(__name__, "life_brick_try.png")
    BALL_BRICK_IMAGE = resource_filename(__name__, "extra_ball_try.png")
    SIZE_BRICK_IMAGE = resource_filename(__name__, "bonus_size_try.png")
    PAD_IMAGE = resource_filename(__name__, "pad.png")
    HIGHSCORE_IMAGE = resource_filename(__name__, "high_score.png")
    MAIN_MENU_IMAGE = resource_filename(__name__, "main_menu.png")
    DOUBLE_PAD_IMAGE = resource_filename(__name__, "double_pad.png")

    SOUND_FILE_HITTING_A_STANDARD_BRICK = resource_filename(__name__, "hitting_a_standard_brick.wav")
    SOUND_FILE_HITTING_SPEED_UP_BRICK = resource_filename(__name__, "hitting_speed_up_brick.wav")
    SOUND_FILE_HITTING_EXTRA_LIFE_BRICK = resource_filename(__name__, "hitting_extra_life_brick.wav")
    SOUND_FILE_BALL_HITTING_A_WALL_OR_A_PAD = resource_filename(__name__, "ball_hitting_a_wall_or_a_pad.wav")
    SOUND_FILE_GAME_OVER = resource_filename(__name__, "game_over.wav")
    SOUND_FILE_HITTING_EXTRA_BALL_BRICK = resource_filename(__name__, "extra_ball.wav")
    SOUND_FILE_HITTING_BONUS_SIZE_BRICK = resource_filename(__name__, "bonus_size.wav")

    SOUND_HIT_BRICK = 0
    SOUND_HIT_SPEED_UP = 1
    SOUND_HIT_EXTRA_LIFE = 2
    SOUND_HIT_WALL_OR_PAD = 3
    SOUND_GAME_OVER = 4
    SOUND_HIT_EXTRA_BALL = 5
    SOUND_HIT_BONUS_SIZE = 6

    PLAYING_SCENE = 0
    HIGHSCORE_SCENE = 1
    MAIN_MENU_SCENE = 2
    GAME_OVER_SCENE = 3
    WIN_SCENE = 4
    CONTROLS_SCENE = 5
    GAME_RULES_SCENE = 6



class GameObject:

    def __init__(self, position, size, image):
        self._position = position
        self._size = size
        self._image = image

    def set_position(self, position):
        self._position = position

    def get_position(self):
        return self._position

    def get_size(self):
        return self._size

    def get_image(self):
        return self._image

    def _intersects_x(self, other):
        other_position = other.get_position()
        other_size = other.get_size()

        if other_position[0] + other_size[0] >= self._position[0] >= other_position[0]:
            return 1
        if other_position[0] + other_size[0] >= self._position[0] + self._size[0] > other_position[0]:
            return 1
        return 0

    def _intersects_y(self, other):
        other_position = other.get_position()
        other_size = other.get_size()

        if other_position[1] + other_size[1] >= self._position[1] >= other_position[1]:
            return 1
        if other_position[1] + other_size[1] >= self._position[1] + self._size[1] > other_position[1]:
            return 1
        return 0

    def intersects(self, other):
        if self._intersects_x(other) and self._intersects_y(other):
            return 1
        return 0




import pygame

from BrickBreaker.Scenes.scene import Scene
from BrickBreaker.Shared import *


class MainMenuScene(Scene):

    def __init__(self, game):
        super().__init__(game)

        self._menu_image = pygame.image.load(GameConstants.MAIN_MENU_IMAGE)

    def render(self):
        self.get_game().screen.blit(self._menu_image, (50, 50))

        self.clear_text()
        self.add_text("F1 - Start Game", x=600, y=280, size=60)
        self.add_text("F2 - Highscore", x=600, y=360, size=60)
        self.add_text("F3 - Controls", x=600, y=440, size=60)
        self.add_text("F4 - Game Rules", x=600, y=520, size=60)
        self.add_text("F5 - Quit", x=600, y=600, size=60)

        super().render()

    def handle_events(self, events):
        super().handle_events(events)

        for event in events:
            if event.type == pygame.QUIT:
                exit()

            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    exit()

                if event.key == pygame.K_F1:
                    self.get_game().change_scene(GameConstants.PLAYING_SCENE)

                if event.key == pygame.K_F2:
                    self.get_game().change_scene(GameConstants.HIGHSCORE_SCENE)

                if event.key == pygame.K_F3:
                    self.get_game().change_scene(GameConstants.CONTROLS_SCENE)

                if event.key == pygame.K_F4:
                    self.get_game().change_scene(GameConstants.GAME_RULES_SCENE)

                if event.key == pygame.K_F5:
                    exit()




import pygame

from BrickBreaker.Scenes.scene import Scene
from BrickBreaker import Highscore
from BrickBreaker.Shared import *


class HighscoreScene(Scene):

    def __init__(self, game):
        super().__init__(game)

        self._high_score_image = pygame.image.load(GameConstants.HIGHSCORE_IMAGE)

    def render(self):

        self.get_game().screen.blit(self._high_score_image, (15, 20))

        self.clear_text()

        highscore = Highscore()

        x = 600
        y = 100
        for position, score in enumerate(highscore.get_scores()):
            self.add_text(f"{position+1}. {score[0]}", x, y, size=45)
            self.add_text(str(score[1]), x + 200, y, size=45)

            y += 30

        self.add_text("F1 - Start Game", x, y + 120, size=60)
        self.add_text("F2 - Main Menu", x, y + 240, size=60)

        super().render()

    def handle_events(self, events):
        super().handle_events(events)

        for event in events:
            if event.type == pygame.QUIT:
                exit()
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_F1:
                    self.get_game().reset()
                    self.get_game().change_scene(GameConstants.PLAYING_SCENE)

                if event.key == pygame.K_F2:
                    self.get_game().change_scene(GameConstants.MAIN_MENU_SCENE)

                if event.key == pygame.K_ESCAPE:
                    self.get_game().change_scene(GameConstants.MAIN_MENU_SCENE)



from BrickBreaker.Scenes.game_over_scene import *
from BrickBreaker.Scenes.highscore_scene import *
from BrickBreaker.Scenes.main_menu_scene import *
from BrickBreaker.Scenes.playing_game_scene import *
from BrickBreaker.Scenes.win_scene import *
from BrickBreaker.Scenes.controls_scene import *
from BrickBreaker.Scenes.game_rules_scene import *
from BrickBreaker.Scenes.scene import *



import pygame

from BrickBreaker.Scenes.scene import Scene
from BrickBreaker.Shared import *


class ControlsScene(Scene):

    def __init__(self, game):
        super().__init__(game)

    def render(self):

        self.clear_text()
        self.add_text("F1 - Use mouse for movement", x=500, y=200, size=60)
        self.add_text("F2 - Use keyboard for movement", x=500, y=280, size=60)
        self.add_text("F3 - Back to Main Menu", x=500, y=360, size=60)

        super().render()

    def handle_events(self, events):
        super().handle_events(events)

        for event in events:
            if event.type == pygame.QUIT:
                exit()

            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    self.get_game().change_scene(GameConstants.MAIN_MENU_SCENE)

                if event.key == pygame.K_F1:
                    self.get_game().get_pad().activate_mouse()

                if event.key == pygame.K_F2:
                    self.get_game().get_pad().activate_keyboard()

                if event.key == pygame.K_F3:
                    self.get_game().change_scene(GameConstants.MAIN_MENU_SCENE)



import pygame

from BrickBreaker.Scenes.scene import Scene
from BrickBreaker.Shared import *
from BrickBreaker import Highscore


class GameOverScene(Scene):

    def __init__(self, game):
        super().__init__(game)

        self._player_name = ""
        self._high_score_image = pygame.image.load(GameConstants.HIGHSCORE_IMAGE)

    def render(self):

        self.get_game().screen.blit(self._high_score_image, (15, 20))

        self.clear_text()
        self.add_text("Please enter your name: ", 300, 200, size=30)
        self.add_text(self._player_name, 300, 250, size=30)

        super().render()

    def handle_events(self, events):
        super().handle_events(events)

        for event in events:
            if event.type == pygame.QUIT:
                exit()

            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_RETURN:
                    game = self.get_game()
                    Highscore().add(self._player_name, game.get_score())
                    game.reset()
                    self._player_name = ""
                    game.change_scene(GameConstants.HIGHSCORE_SCENE)
                elif 122 >= event.key >= 65:
                    self._player_name += chr(event.key)
                elif event.key == pygame.K_BACKSPACE:
                    self._player_name = self._player_name[:-1]

                if event.key == pygame.K_F1:
                    self.get_game().reset()
                    self.get_game().change_scene(GameConstants.PLAYING_SCENE)



import random

from BrickBreaker.Bricks import *
from BrickBreaker.Shared import *

import pygame


class Level:

    def __init__(self, game):
        self._game = game
        self._bricks = []
        self._amount_of_bricks_left = 0
        self._current_level = 1

    def get_bricks(self):
        return self._bricks

    def get_game(self):
        return self._game

    def get_amount_of_bricks_left(self):
        return self._amount_of_bricks_left

    def brick_hit(self):
        self._amount_of_bricks_left -= 1

    def load_next_level(self):
        self._current_level += 1
        self.load_random()

    def load_random(self):

        if self._current_level >= 9:
            self.get_game().change_scene(GameConstants.WIN_SCENE)

        self._bricks = []

        x, y = 0, 0

        max_bricks = int(GameConstants.SCREEN_SIZE[0] / GameConstants.BRICK_SIZE[0])
        rows = self._current_level

        for row in range(0, rows):
            amount_of_life_bricks = 0
            amount_of_speed_bricks = 0
            amount_of_ball_bricks = 0
            amount_of_size_bricks = 0

            for brick in range(0, max_bricks):
                brick_type = random.randint(2, 5)

                if brick % 2 == 1:

                    if brick_type == 2 and amount_of_speed_bricks < self._current_level * 2:
                        brick = SpeedBrick([x, y], pygame.image.load(GameConstants.SPEED_BRICK_IMAGE), self._game)
                        self._bricks.append(brick)
                        self._amount_of_bricks_left += 1
                        amount_of_speed_bricks += 1

                    elif brick_type == 3 and amount_of_life_bricks < self._current_level * 2:
                        brick = LifeBrick([x, y], pygame.image.load(GameConstants.LIFE_BRICK_IMAGE), self._game)
                        self._bricks.append(brick)
                        self._amount_of_bricks_left += 1
                        amount_of_life_bricks += 1

                    elif brick_type == 4 and amount_of_size_bricks < self._current_level * 2:
                        brick = SizeBrick([x, y], pygame.image.load(GameConstants.SIZE_BRICK_IMAGE), self._game)
                        self._bricks.append(brick)
                        self._amount_of_bricks_left += 1
                        amount_of_size_bricks += 1

                    elif brick_type == 5 and amount_of_ball_bricks < self._current_level * 2:
                        brick = BallBrick([x, y], pygame.image.load(GameConstants.BALL_BRICK_IMAGE), self._game)
                        self._bricks.append(brick)
                        self._amount_of_bricks_left += 1
                        amount_of_ball_bricks += 1
                    else:
                        brick = Brick([x, y], pygame.image.load(GameConstants.STANDARD_BRICK_IMAGE), self._game)
                        self._bricks.append(brick)
                        self._amount_of_bricks_left += 1

                else:
                    brick = Brick([x, y], pygame.image.load(GameConstants.STANDARD_BRICK_IMAGE), self._game)
                    self._bricks.append(brick)
                    self._amount_of_bricks_left += 1

                x += GameConstants.BRICK_SIZE[0]

            x = 0
            y += GameConstants.BRICK_SIZE[1]

